@@ -38,13 +38,10 @@
 ****************************************************************************/
 
 #include "qv4baselinejit_p.h"
-#include "qv4jithelpers_p.h"
 #include "qv4baselineassembler_p.h"
 #include <private/qv4lookup_p.h>
 #include <private/qv4generatorobject_p.h>
 
-#ifdef V4_ENABLE_JIT
-
 QT_USE_NAMESPACE
 using namespace QV4;
 using namespace QV4::JIT;
@@ -52,7 +49,7 @@ using namespace QV4::Moth;
 
 BaselineJIT::BaselineJIT(Function *function)
     : function(function)
-    , as(new BaselineAssembler(function->compilationUnit->constants))
+      , as(new BaselineAssembler(&(function->compilationUnit->constants->asValue<Value>())))
 {}
 
 BaselineJIT::~BaselineJIT()
@@ -63,9 +60,13 @@ void BaselineJIT::generate()
 //    qDebug()<<"jitting" << function->name()->toQString();
     const char *code = function->codeData;
     uint len = function->compiledFunction->codeSize;
-    labels = collectLabelsInBytecode(code, len);
+
+    for (unsigned i = 0, ei = function->compiledFunction->nLabelInfos; i != ei; ++i)
+        labels.insert(int(function->compiledFunction->labelInfoTable()[i]));
 
     as->generatePrologue();
+    // Make sure the ACC register is initialized and not clobbered by the caller.
+    as->loadAccumulatorFromFrame();
     decode(code, len);
     as->generateEpilogue();
 
@@ -75,10 +76,12 @@ void BaselineJIT::generate()
 
 #define STORE_IP() as->storeInstructionPointer(nextInstructionOffset())
 #define STORE_ACC() as->saveAccumulatorInFrame()
-#define BASELINEJIT_GENERATE_RUNTIME_CALL(function, destination) \
-    as->GENERATE_RUNTIME_CALL(function, destination)
-#define BASELINEJIT_GENERATE_TAIL_CALL(function) \
-    as->GENERATE_TAIL_CALL(function)
+#define LOAD_ACC() as->loadAccumulatorFromFrame()
+#define BASELINEJIT_GENERATE_RUNTIME_CALL(function, destination) { \
+    as->GENERATE_RUNTIME_CALL(function, destination); \
+    if (Runtime::function::throws) \
+        as->checkException(); \
+    else {} } // this else prevents else statements after the macro from attaching to the if above
 
 void BaselineJIT::generate_Ret()
 {
@@ -181,7 +184,7 @@ void BaselineJIT::generate_MoveRegExp(int regExpId, int destReg)
     as->prepareCallWithArgCount(2);
     as->passInt32AsArg(regExpId, 1);
     as->passEngineAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Runtime::method_regexpLiteral, CallResultDestination::InAccumulator);
+    BASELINEJIT_GENERATE_RUNTIME_CALL(RegexpLiteral, CallResultDestination::InAccumulator);
     as->storeReg(destReg);
 }
 
@@ -190,7 +193,7 @@ void BaselineJIT::generate_LoadClosure(int value)
     as->prepareCallWithArgCount(2);
     as->passInt32AsArg(value, 1);
     as->passEngineAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Runtime::method_closure, CallResultDestination::InAccumulator);
+    BASELINEJIT_GENERATE_RUNTIME_CALL(Closure, CallResultDestination::InAccumulator);
 }
 
 void BaselineJIT::generate_LoadName(int name)
@@ -199,28 +202,24 @@ void BaselineJIT::generate_LoadName(int name)
     as->prepareCallWithArgCount(2);
     as->passInt32AsArg(name, 1);
     as->passEngineAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Runtime::method_loadName, CallResultDestination::InAccumulator);
-    as->checkException();
+    BASELINEJIT_GENERATE_RUNTIME_CALL(LoadName, CallResultDestination::InAccumulator);
 }
 
 void BaselineJIT::generate_LoadGlobalLookup(int index)
 {
     as->prepareCallWithArgCount(3);
     as->passInt32AsArg(index, 2);
-    as->passEngineAsArg(1);
-    as->passFunctionAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Helpers::loadGlobalLookup, CallResultDestination::InAccumulator);
-    as->checkException();
+    as->passFunctionAsArg(1);
+    as->passEngineAsArg(0);
+    BASELINEJIT_GENERATE_RUNTIME_CALL(LoadGlobalLookup, CallResultDestination::InAccumulator);
 }
 
 void BaselineJIT::generate_LoadQmlContextPropertyLookup(int index)
 {
-    as->prepareCallWithArgCount(3);
-    as->passInt32AsArg(index, 2);
-    as->passEngineAsArg(1);
-    as->passFunctionAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Helpers::loadQmlContextPropertyLookup, CallResultDestination::InAccumulator);
-    as->checkException();
+    as->prepareCallWithArgCount(2);
+    as->passInt32AsArg(index, 1);
+    as->passEngineAsArg(0);
+    BASELINEJIT_GENERATE_RUNTIME_CALL(LoadQmlContextPropertyLookup, CallResultDestination::InAccumulator);
 }
 
 void BaselineJIT::generate_StoreNameSloppy(int name)
@@ -231,8 +230,8 @@ void BaselineJIT::generate_StoreNameSloppy(int name)
     as->passAccumulatorAsArg(2);
     as->passInt32AsArg(name, 1);
     as->passEngineAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Runtime::method_storeNameSloppy, CallResultDestination::Ignore);
-    as->checkException();
+    BASELINEJIT_GENERATE_RUNTIME_CALL(StoreNameSloppy, CallResultDestination::Ignore);
+    LOAD_ACC();
 }
 
 void BaselineJIT::generate_StoreNameStrict(int name)
@@ -243,8 +242,8 @@ void BaselineJIT::generate_StoreNameStrict(int name)
     as->passAccumulatorAsArg(2);
     as->passInt32AsArg(name, 1);
     as->passEngineAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Runtime::method_storeNameStrict, CallResultDestination::Ignore);
-    as->checkException();
+    BASELINEJIT_GENERATE_RUNTIME_CALL(StoreNameStrict, CallResultDestination::Ignore);
+    LOAD_ACC();
 }
 
 void BaselineJIT::generate_LoadElement(int base)
@@ -255,8 +254,7 @@ void BaselineJIT::generate_LoadElement(int base)
     as->passAccumulatorAsArg(2);
     as->passJSSlotAsArg(base, 1);
     as->passEngineAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Runtime::method_loadElement, CallResultDestination::InAccumulator);
-    as->checkException();
+    BASELINEJIT_GENERATE_RUNTIME_CALL(LoadElement, CallResultDestination::InAccumulator);
 }
 
 void BaselineJIT::generate_StoreElement(int base, int index)
@@ -268,8 +266,8 @@ void BaselineJIT::generate_StoreElement(int base, int index)
     as->passJSSlotAsArg(index, 2);
     as->passJSSlotAsArg(base, 1);
     as->passEngineAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Runtime::method_storeElement, CallResultDestination::Ignore);
-    as->checkException();
+    BASELINEJIT_GENERATE_RUNTIME_CALL(StoreElement, CallResultDestination::Ignore);
+    LOAD_ACC();
 }
 
 void BaselineJIT::generate_LoadProperty(int name)
@@ -280,8 +278,7 @@ void BaselineJIT::generate_LoadProperty(int name)
     as->passInt32AsArg(name, 2);
     as->passAccumulatorAsArg(1);
     as->passEngineAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Runtime::method_loadProperty, CallResultDestination::InAccumulator);
-    as->checkException();
+    BASELINEJIT_GENERATE_RUNTIME_CALL(LoadProperty, CallResultDestination::InAccumulator);
 }
 
 void BaselineJIT::generate_GetLookup(int index)
@@ -291,10 +288,9 @@ void BaselineJIT::generate_GetLookup(int index)
     as->prepareCallWithArgCount(4);
     as->passInt32AsArg(index, 3);
     as->passAccumulatorAsArg(2);
-    as->passEngineAsArg(1);
-    as->passFunctionAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Helpers::getLookup, CallResultDestination::InAccumulator);
-    as->checkException();
+    as->passFunctionAsArg(1);
+    as->passEngineAsArg(0);
+    BASELINEJIT_GENERATE_RUNTIME_CALL(GetLookup, CallResultDestination::InAccumulator);
 }
 
 void BaselineJIT::generate_StoreProperty(int name, int base)
@@ -306,8 +302,8 @@ void BaselineJIT::generate_StoreProperty(int name, int base)
     as->passInt32AsArg(name, 2);
     as->passJSSlotAsArg(base, 1);
     as->passEngineAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Runtime::method_storeProperty, CallResultDestination::Ignore);
-    as->checkException();
+    BASELINEJIT_GENERATE_RUNTIME_CALL(StoreProperty, CallResultDestination::Ignore);
+    LOAD_ACC();
 }
 
 void BaselineJIT::generate_SetLookup(int index, int base)
@@ -316,23 +312,22 @@ void BaselineJIT::generate_SetLookup(int index, int base)
     STORE_ACC();
     as->prepareCallWithArgCount(4);
     as->passAccumulatorAsArg(3);
-    as->passJSSlotAsArg(base, 2);
-    as->passInt32AsArg(index, 1);
+    as->passInt32AsArg(index, 2);
+    as->passJSSlotAsArg(base, 1);
     as->passFunctionAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL((function->isStrict() ? Helpers::setLookupStrict : Helpers::setLookupSloppy),
-                                      CallResultDestination::InAccumulator);
-    as->checkException();
+    if (function->isStrict())
+        BASELINEJIT_GENERATE_RUNTIME_CALL(SetLookupStrict, CallResultDestination::InAccumulator)
+    else
+        BASELINEJIT_GENERATE_RUNTIME_CALL(SetLookupSloppy, CallResultDestination::InAccumulator)
 }
 
 void BaselineJIT::generate_LoadSuperProperty(int property)
 {
     STORE_IP();
-    STORE_ACC();
     as->prepareCallWithArgCount(2);
     as->passJSSlotAsArg(property, 1);
     as->passEngineAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Runtime::method_loadSuperProperty, CallResultDestination::InAccumulator);
-    as->checkException();
+    BASELINEJIT_GENERATE_RUNTIME_CALL(LoadSuperProperty, CallResultDestination::InAccumulator);
 }
 
 void BaselineJIT::generate_StoreSuperProperty(int property)
@@ -343,8 +338,8 @@ void BaselineJIT::generate_StoreSuperProperty(int property)
     as->passAccumulatorAsArg(2);
     as->passJSSlotAsArg(property, 1);
     as->passEngineAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Runtime::method_storeSuperProperty, CallResultDestination::Ignore);
-    as->checkException();
+    BASELINEJIT_GENERATE_RUNTIME_CALL(StoreSuperProperty, CallResultDestination::Ignore);
+    LOAD_ACC();
 }
 
 void BaselineJIT::generate_Yield()
@@ -373,8 +368,7 @@ void BaselineJIT::generate_CallValue(int name, int argc, int argv)
     as->passJSSlotAsArg(argv, 2);
     as->passJSSlotAsArg(name, 1);
     as->passEngineAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Runtime::method_callValue, CallResultDestination::InAccumulator);
-    as->checkException();
+    BASELINEJIT_GENERATE_RUNTIME_CALL(CallValue, CallResultDestination::InAccumulator);
 }
 
 void BaselineJIT::generate_CallWithReceiver(int name, int thisObject, int argc, int argv)
@@ -386,8 +380,7 @@ void BaselineJIT::generate_CallWithReceiver(int name, int thisObject, int argc,
     as->passJSSlotAsArg(thisObject, 2);
     as->passJSSlotAsArg(name, 1);
     as->passEngineAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Runtime::method_callWithReceiver, CallResultDestination::InAccumulator);
-    as->checkException();
+    BASELINEJIT_GENERATE_RUNTIME_CALL(CallWithReceiver, CallResultDestination::InAccumulator);
 }
 
 void BaselineJIT::generate_CallProperty(int name, int base, int argc, int argv)
@@ -399,8 +392,7 @@ void BaselineJIT::generate_CallProperty(int name, int base, int argc, int argv)
     as->passInt32AsArg(name, 2);
     as->passJSSlotAsArg(base, 1);
     as->passEngineAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Runtime::method_callProperty, CallResultDestination::InAccumulator);
-    as->checkException();
+    BASELINEJIT_GENERATE_RUNTIME_CALL(CallProperty, CallResultDestination::InAccumulator);
 }
 
 void BaselineJIT::generate_CallPropertyLookup(int lookupIndex, int base, int argc, int argv)
@@ -412,8 +404,7 @@ void BaselineJIT::generate_CallPropertyLookup(int lookupIndex, int base, int arg
     as->passInt32AsArg(lookupIndex, 2);
     as->passJSSlotAsArg(base, 1);
     as->passEngineAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Runtime::method_callPropertyLookup, CallResultDestination::InAccumulator);
-    as->checkException();
+    BASELINEJIT_GENERATE_RUNTIME_CALL(CallPropertyLookup, CallResultDestination::InAccumulator);
 }
 
 void BaselineJIT::generate_CallElement(int base, int index, int argc, int argv)
@@ -425,8 +416,7 @@ void BaselineJIT::generate_CallElement(int base, int index, int argc, int argv)
     as->passJSSlotAsArg(index, 2);
     as->passJSSlotAsArg(base, 1);
     as->passEngineAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Runtime::method_callElement, CallResultDestination::InAccumulator);
-    as->checkException();
+    BASELINEJIT_GENERATE_RUNTIME_CALL(CallElement, CallResultDestination::InAccumulator);
 }
 
 void BaselineJIT::generate_CallName(int name, int argc, int argv)
@@ -437,8 +427,7 @@ void BaselineJIT::generate_CallName(int name, int argc, int argv)
     as->passJSSlotAsArg(argv, 2);
     as->passInt32AsArg(name, 1);
     as->passEngineAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Runtime::method_callName, CallResultDestination::InAccumulator);
-    as->checkException();
+    BASELINEJIT_GENERATE_RUNTIME_CALL(CallName, CallResultDestination::InAccumulator);
 }
 
 void BaselineJIT::generate_CallPossiblyDirectEval(int argc, int argv)
@@ -448,8 +437,7 @@ void BaselineJIT::generate_CallPossiblyDirectEval(int argc, int argv)
     as->passInt32AsArg(argc, 2);
     as->passJSSlotAsArg(argv, 1);
     as->passEngineAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Runtime::method_callPossiblyDirectEval, CallResultDestination::InAccumulator);
-    as->checkException();
+    BASELINEJIT_GENERATE_RUNTIME_CALL(CallPossiblyDirectEval, CallResultDestination::InAccumulator);
 }
 
 void BaselineJIT::generate_CallGlobalLookup(int index, int argc, int argv)
@@ -460,8 +448,7 @@ void BaselineJIT::generate_CallGlobalLookup(int index, int argc, int argv)
     as->passJSSlotAsArg(argv, 2);
     as->passInt32AsArg(index, 1);
     as->passEngineAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Runtime::method_callGlobalLookup, CallResultDestination::InAccumulator);
-    as->checkException();
+    BASELINEJIT_GENERATE_RUNTIME_CALL(CallGlobalLookup, CallResultDestination::InAccumulator);
 }
 
 void BaselineJIT::generate_CallQmlContextPropertyLookup(int index, int argc, int argv)
@@ -472,8 +459,7 @@ void BaselineJIT::generate_CallQmlContextPropertyLookup(int index, int argc, int
     as->passJSSlotAsArg(argv, 2);
     as->passInt32AsArg(index, 1);
     as->passEngineAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Runtime::method_callQmlContextPropertyLookup, CallResultDestination::InAccumulator);
-    as->checkException();
+    BASELINEJIT_GENERATE_RUNTIME_CALL(CallQmlContextPropertyLookup, CallResultDestination::InAccumulator);
 }
 
 void BaselineJIT::generate_CallWithSpread(int func, int thisObject, int argc, int argv)
@@ -485,8 +471,7 @@ void BaselineJIT::generate_CallWithSpread(int func, int thisObject, int argc, in
     as->passJSSlotAsArg(thisObject, 2);
     as->passJSSlotAsArg(func, 1);
     as->passEngineAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Runtime::method_callWithSpread, CallResultDestination::InAccumulator);
-    as->checkException();
+    BASELINEJIT_GENERATE_RUNTIME_CALL(CallWithSpread, CallResultDestination::InAccumulator);
 }
 
 void BaselineJIT::generate_TailCall(int func, int thisObject, int argc, int argv)
@@ -505,8 +490,7 @@ void BaselineJIT::generate_Construct(int func, int argc, int argv)
     as->passAccumulatorAsArg(2);
     as->passJSSlotAsArg(func, 1);
     as->passEngineAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Runtime::method_construct, CallResultDestination::InAccumulator);
-    as->checkException();
+    BASELINEJIT_GENERATE_RUNTIME_CALL(Construct, CallResultDestination::InAccumulator);
 }
 
 void BaselineJIT::generate_ConstructWithSpread(int func, int argc, int argv)
@@ -519,14 +503,13 @@ void BaselineJIT::generate_ConstructWithSpread(int func, int argc, int argv)
     as->passAccumulatorAsArg(2);
     as->passJSSlotAsArg(func, 1);
     as->passEngineAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Runtime::method_constructWithSpread, CallResultDestination::InAccumulator);
-    as->checkException();
+    BASELINEJIT_GENERATE_RUNTIME_CALL(ConstructWithSpread, CallResultDestination::InAccumulator);
 }
 
 void BaselineJIT::generate_SetUnwindHandler(int offset)
 {
     if (offset)
-        as->setUnwindHandler(absoluteOffsetForJump(offset));
+        labels.insert(as->setUnwindHandler(absoluteOffset(offset)));
     else
         as->clearUnwindHandler();
 }
@@ -538,7 +521,7 @@ void BaselineJIT::generate_UnwindDispatch()
 
 void BaselineJIT::generate_UnwindToLabel(int level, int offset)
 {
-    as->unwindToLabel(level, absoluteOffsetForJump(offset));
+    labels.insert(as->unwindToLabel(level, absoluteOffset(offset)));
 }
 
 void BaselineJIT::generate_DeadTemporalZoneCheck(int name)
@@ -553,7 +536,7 @@ void BaselineJIT::generate_ThrowException()
     as->prepareCallWithArgCount(2);
     as->passAccumulatorAsArg(1);
     as->passEngineAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Runtime::method_throwException, CallResultDestination::Ignore);
+    BASELINEJIT_GENERATE_RUNTIME_CALL(ThrowException, CallResultDestination::Ignore);
     as->gotoCatchException();
 }
 
@@ -564,8 +547,7 @@ void BaselineJIT::generate_CreateCallContext()
 {
     as->prepareCallWithArgCount(1);
     as->passCppFrameAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(ExecutionContext::newCallContext, CallResultDestination::Ignore); // keeps result in return value register
-    as->storeHeapObject(CallData::Context);
+    BASELINEJIT_GENERATE_RUNTIME_CALL(PushCallContext, CallResultDestination::Ignore);
 }
 
 void BaselineJIT::generate_PushCatchContext(int index, int name) { as->pushCatchContext(index, name); }
@@ -575,11 +557,9 @@ void BaselineJIT::generate_PushWithContext()
     STORE_IP();
     as->saveAccumulatorInFrame();
     as->prepareCallWithArgCount(2);
-    as->passJSSlotAsArg(0, 1);
+    as->passJSSlotAsArg(CallData::Accumulator, 1);
     as->passEngineAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Runtime::method_createWithContext, CallResultDestination::Ignore);  // keeps result in return value register
-    as->checkException();
-    as->storeHeapObject(CallData::Context);
+    BASELINEJIT_GENERATE_RUNTIME_CALL(PushWithContext, CallResultDestination::InAccumulator);
 }
 
 void BaselineJIT::generate_PushBlockContext(int index)
@@ -587,35 +567,37 @@ void BaselineJIT::generate_PushBlockContext(int index)
     as->saveAccumulatorInFrame();
     as->prepareCallWithArgCount(2);
     as->passInt32AsArg(index, 1);
-    as->passJSSlotAsArg(0, 0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Helpers::pushBlockContext, CallResultDestination::Ignore);
+    as->passEngineAsArg(0);
+    BASELINEJIT_GENERATE_RUNTIME_CALL(PushBlockContext, CallResultDestination::Ignore);
+    as->loadAccumulatorFromFrame();
 }
 
 void BaselineJIT::generate_CloneBlockContext()
 {
     as->saveAccumulatorInFrame();
     as->prepareCallWithArgCount(1);
-    as->passJSSlotAsArg(CallData::Context, 0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Helpers::cloneBlockContext, CallResultDestination::Ignore);
+    as->passEngineAsArg(0);
+    BASELINEJIT_GENERATE_RUNTIME_CALL(CloneBlockContext, CallResultDestination::Ignore);
+    as->loadAccumulatorFromFrame();
 }
 
 void BaselineJIT::generate_PushScriptContext(int index)
 {
     as->saveAccumulatorInFrame();
-    as->prepareCallWithArgCount(3);
-    as->passInt32AsArg(index, 2);
-    as->passEngineAsArg(1);
-    as->passJSSlotAsArg(0, 0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Helpers::pushScriptContext, CallResultDestination::Ignore);
+    as->prepareCallWithArgCount(2);
+    as->passInt32AsArg(index, 1);
+    as->passEngineAsArg(0);
+    BASELINEJIT_GENERATE_RUNTIME_CALL(PushScriptContext, CallResultDestination::Ignore);
+    as->loadAccumulatorFromFrame();
 }
 
 void BaselineJIT::generate_PopScriptContext()
 {
     as->saveAccumulatorInFrame();
-    as->prepareCallWithArgCount(2);
-    as->passEngineAsArg(1);
-    as->passJSSlotAsArg(0, 0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Helpers::popScriptContext, CallResultDestination::Ignore);
+    as->prepareCallWithArgCount(1);
+    as->passEngineAsArg(0);
+    BASELINEJIT_GENERATE_RUNTIME_CALL(PopScriptContext, CallResultDestination::Ignore);
+    as->loadAccumulatorFromFrame();
 }
 
 void BaselineJIT::generate_PopContext() { as->popContext(); }
@@ -627,8 +609,7 @@ void BaselineJIT::generate_GetIterator(int iterator)
     as->passInt32AsArg(iterator, 2);
     as->passAccumulatorAsArg(1);
     as->passEngineAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Runtime::method_getIterator, CallResultDestination::InAccumulator);
-    as->checkException();
+    BASELINEJIT_GENERATE_RUNTIME_CALL(GetIterator, CallResultDestination::InAccumulator);
 }
 
 void BaselineJIT::generate_IteratorNext(int value, int done)
@@ -638,9 +619,8 @@ void BaselineJIT::generate_IteratorNext(int value, int done)
     as->passJSSlotAsArg(value, 2);
     as->passAccumulatorAsArg(1);
     as->passEngineAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Runtime::method_iteratorNext, CallResultDestination::InAccumulator);
+    BASELINEJIT_GENERATE_RUNTIME_CALL(IteratorNext, CallResultDestination::InAccumulator);
     as->storeReg(done);
-    as->checkException();
 }
 
 void BaselineJIT::generate_IteratorNextForYieldStar(int iterator, int object)
@@ -651,8 +631,7 @@ void BaselineJIT::generate_IteratorNextForYieldStar(int iterator, int object)
     as->passJSSlotAsArg(iterator, 2);
     as->passAccumulatorAsArg(1);
     as->passEngineAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Runtime::method_iteratorNextForYieldStar, CallResultDestination::InAccumulator);
-    as->checkException();
+    BASELINEJIT_GENERATE_RUNTIME_CALL(IteratorNextForYieldStar, CallResultDestination::InAccumulator);
 }
 
 void BaselineJIT::generate_IteratorClose(int done)
@@ -662,8 +641,7 @@ void BaselineJIT::generate_IteratorClose(int done)
     as->passJSSlotAsArg(done, 2);
     as->passAccumulatorAsArg(1);
     as->passEngineAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Runtime::method_iteratorClose, CallResultDestination::InAccumulator);
-    as->checkException();
+    BASELINEJIT_GENERATE_RUNTIME_CALL(IteratorClose, CallResultDestination::InAccumulator);
 }
 
 void BaselineJIT::generate_DestructureRestElement()
@@ -672,29 +650,28 @@ void BaselineJIT::generate_DestructureRestElement()
     as->prepareCallWithArgCount(2);
     as->passAccumulatorAsArg(1);
     as->passEngineAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Runtime::method_destructureRestElement, CallResultDestination::InAccumulator);
-    as->checkException();
+    BASELINEJIT_GENERATE_RUNTIME_CALL(DestructureRestElement, CallResultDestination::InAccumulator);
 }
 
 void BaselineJIT::generate_DeleteProperty(int base, int index)
 {
     STORE_IP();
-    as->prepareCallWithArgCount(3);
-    as->passJSSlotAsArg(index, 2);
-    as->passJSSlotAsArg(base, 1);
-    as->passFunctionAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Helpers::deleteProperty, CallResultDestination::InAccumulator);
-    as->checkException();
+    as->prepareCallWithArgCount(4);
+    as->passJSSlotAsArg(index, 3);
+    as->passJSSlotAsArg(base, 2);
+    as->passFunctionAsArg(1);
+    as->passEngineAsArg(0);
+    BASELINEJIT_GENERATE_RUNTIME_CALL(DeleteProperty, CallResultDestination::InAccumulator);
 }
 
 void BaselineJIT::generate_DeleteName(int name)
 {
     STORE_IP();
-    as->prepareCallWithArgCount(2);
-    as->passInt32AsArg(name, 1);
-    as->passFunctionAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Helpers::deleteName, CallResultDestination::InAccumulator);
-    as->checkException();
+    as->prepareCallWithArgCount(3);
+    as->passInt32AsArg(name, 2);
+    as->passFunctionAsArg(1);
+    as->passEngineAsArg(0);
+    BASELINEJIT_GENERATE_RUNTIME_CALL(DeleteName, CallResultDestination::InAccumulator);
 }
 
 void BaselineJIT::generate_TypeofName(int name)
@@ -702,7 +679,7 @@ void BaselineJIT::generate_TypeofName(int name)
     as->prepareCallWithArgCount(2);
     as->passInt32AsArg(name, 1);
     as->passEngineAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Runtime::method_typeofName, CallResultDestination::InAccumulator);
+    BASELINEJIT_GENERATE_RUNTIME_CALL(TypeofName, CallResultDestination::InAccumulator);
 }
 
 void BaselineJIT::generate_TypeofValue()
@@ -711,16 +688,18 @@ void BaselineJIT::generate_TypeofValue()
     as->prepareCallWithArgCount(2);
     as->passAccumulatorAsArg(1);
     as->passEngineAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Runtime::method_typeofValue, CallResultDestination::InAccumulator);
+    BASELINEJIT_GENERATE_RUNTIME_CALL(TypeofValue, CallResultDestination::InAccumulator);
 }
 
 void BaselineJIT::generate_DeclareVar(int varName, int isDeletable)
 {
+    STORE_ACC();
     as->prepareCallWithArgCount(3);
     as->passInt32AsArg(varName, 2);
     as->passInt32AsArg(isDeletable, 1);
     as->passEngineAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Runtime::method_declareVar, CallResultDestination::Ignore);
+    BASELINEJIT_GENERATE_RUNTIME_CALL(DeclareVar, CallResultDestination::Ignore);
+    LOAD_ACC();
 }
 
 void BaselineJIT::generate_DefineArray(int argc, int args)
@@ -729,7 +708,7 @@ void BaselineJIT::generate_DefineArray(int argc, int args)
     as->passInt32AsArg(argc, 2);
     as->passJSSlotAsArg(args, 1);
     as->passEngineAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Runtime::method_arrayLiteral, CallResultDestination::InAccumulator);
+    BASELINEJIT_GENERATE_RUNTIME_CALL(ArrayLiteral, CallResultDestination::InAccumulator);
 }
 
 void BaselineJIT::generate_DefineObjectLiteral(int internalClassId, int argc, int args)
@@ -739,7 +718,7 @@ void BaselineJIT::generate_DefineObjectLiteral(int internalClassId, int argc, in
     as->passJSSlotAsArg(args, 2);
     as->passInt32AsArg(internalClassId, 1);
     as->passEngineAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Runtime::method_objectLiteral, CallResultDestination::InAccumulator);
+    BASELINEJIT_GENERATE_RUNTIME_CALL(ObjectLiteral, CallResultDestination::InAccumulator);
 }
 
 void BaselineJIT::generate_CreateClass(int classIndex, int heritage, int computedNames)
@@ -749,14 +728,14 @@ void BaselineJIT::generate_CreateClass(int classIndex, int heritage, int compute
     as->passJSSlotAsArg(heritage, 2);
     as->passInt32AsArg(classIndex, 1);
     as->passEngineAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Runtime::method_createClass, CallResultDestination::InAccumulator);
+    BASELINEJIT_GENERATE_RUNTIME_CALL(CreateClass, CallResultDestination::InAccumulator);
 }
 
 void BaselineJIT::generate_CreateMappedArgumentsObject()
 {
     as->prepareCallWithArgCount(1);
     as->passEngineAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Runtime::method_createMappedArgumentsObject,
+    BASELINEJIT_GENERATE_RUNTIME_CALL(CreateMappedArgumentsObject,
                               CallResultDestination::InAccumulator);
 }
 
@@ -764,7 +743,7 @@ void BaselineJIT::generate_CreateUnmappedArgumentsObject()
 {
     as->prepareCallWithArgCount(1);
     as->passEngineAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Runtime::method_createUnmappedArgumentsObject,
+    BASELINEJIT_GENERATE_RUNTIME_CALL(CreateUnmappedArgumentsObject,
                               CallResultDestination::InAccumulator);
 }
 
@@ -773,16 +752,18 @@ void BaselineJIT::generate_CreateRestParameter(int argIndex)
     as->prepareCallWithArgCount(2);
     as->passInt32AsArg(argIndex, 1);
     as->passEngineAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Runtime::method_createRestParameter, CallResultDestination::InAccumulator);
+    BASELINEJIT_GENERATE_RUNTIME_CALL(CreateRestParameter, CallResultDestination::InAccumulator);
 }
 
 void BaselineJIT::generate_ConvertThisToObject()
 {
+    STORE_ACC();
     as->prepareCallWithArgCount(2);
     as->passJSSlotAsArg(CallData::This, 1);
     as->passEngineAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Helpers::convertThisToObject, CallResultDestination::Ignore);
-    as->checkException();
+    BASELINEJIT_GENERATE_RUNTIME_CALL(ConvertThisToObject, CallResultDestination::InAccumulator);
+    as->storeReg(CallData::This);
+    LOAD_ACC();
 }
 
 void BaselineJIT::generate_LoadSuperConstructor()
@@ -790,8 +771,7 @@ void BaselineJIT::generate_LoadSuperConstructor()
     as->prepareCallWithArgCount(2);
     as->passJSSlotAsArg(CallData::Function, 1);
     as->passEngineAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Runtime::method_loadSuperConstructor, CallResultDestination::InAccumulator);
-    as->checkException();
+    BASELINEJIT_GENERATE_RUNTIME_CALL(LoadSuperConstructor, CallResultDestination::InAccumulator);
 }
 
 void BaselineJIT::generate_ToObject()
@@ -800,16 +780,39 @@ void BaselineJIT::generate_ToObject()
     as->prepareCallWithArgCount(2);
     as->passAccumulatorAsArg(1);
     as->passEngineAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Helpers::toObject, CallResultDestination::InAccumulator);
-    as->checkException();
+    BASELINEJIT_GENERATE_RUNTIME_CALL(ToObject, CallResultDestination::InAccumulator);
 
 }
 
-void BaselineJIT::generate_Jump(int offset) { as->jump(absoluteOffsetForJump(offset)); }
-void BaselineJIT::generate_JumpTrue(int offset) { as->jumpTrue(absoluteOffsetForJump(offset)); }
-void BaselineJIT::generate_JumpFalse(int offset) { as->jumpFalse(absoluteOffsetForJump(offset)); }
-void BaselineJIT::generate_JumpNoException(int offset) { as->jumpNoException(absoluteOffsetForJump(offset)); }
-void BaselineJIT::generate_JumpNotUndefined(int offset) { as->jumpNotUndefined(absoluteOffsetForJump(offset)); }
+void BaselineJIT::generate_Jump(int offset)
+{
+    labels.insert(as->jump(absoluteOffset(offset)));
+}
+
+void BaselineJIT::generate_JumpTrue(int offset)
+{
+    labels.insert(as->jumpTrue(absoluteOffset(offset)));
+}
+
+void BaselineJIT::generate_JumpFalse(int offset)
+{
+    labels.insert(as->jumpFalse(absoluteOffset(offset)));
+}
+
+void BaselineJIT::generate_JumpNoException(int offset)
+{
+    labels.insert(as->jumpNoException(absoluteOffset(offset)));
+}
+
+void BaselineJIT::generate_JumpNotUndefined(int offset)
+{
+    labels.insert(as->jumpNotUndefined(absoluteOffset(offset)));
+}
+
+void BaselineJIT::generate_CheckException()
+{
+    as->checkException();
+}
 
 void BaselineJIT::generate_CmpEqNull() { as->cmpeqNull(); }
 void BaselineJIT::generate_CmpNeNull() { as->cmpneNull(); }
@@ -831,8 +834,7 @@ void BaselineJIT::generate_CmpIn(int lhs)
     as->passAccumulatorAsArg(2);
     as->passJSSlotAsArg(lhs, 1);
     as->passEngineAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Runtime::method_in, CallResultDestination::InAccumulator);
-    as->checkException();
+    BASELINEJIT_GENERATE_RUNTIME_CALL(In, CallResultDestination::InAccumulator);
 }
 
 void BaselineJIT::generate_CmpInstanceOf(int lhs)
@@ -842,8 +844,7 @@ void BaselineJIT::generate_CmpInstanceOf(int lhs)
     as->passAccumulatorAsArg(2);
     as->passJSSlotAsArg(lhs, 1);
     as->passEngineAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Runtime::method_instanceof, CallResultDestination::InAccumulator);
-    as->checkException();
+    BASELINEJIT_GENERATE_RUNTIME_CALL(Instanceof, CallResultDestination::InAccumulator);
 }
 
 void BaselineJIT::generate_UNot() { as->unot(); }
@@ -874,8 +875,7 @@ void BaselineJIT::generate_Exp(int lhs) {
     as->prepareCallWithArgCount(2);
     as->passAccumulatorAsArg(1);
     as->passJSSlotAsArg(lhs, 0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Helpers::exp, CallResultDestination::InAccumulator);
-    as->checkException();
+    BASELINEJIT_GENERATE_RUNTIME_CALL(Exp, CallResultDestination::InAccumulator);
 }
 void BaselineJIT::generate_Mul(int lhs) { as->mul(lhs); }
 void BaselineJIT::generate_Div(int lhs) { as->div(lhs); }
@@ -907,8 +907,8 @@ void BaselineJIT::generate_ThrowOnNullOrUndefined()
     as->prepareCallWithArgCount(2);
     as->passAccumulatorAsArg(1);
     as->passEngineAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(Helpers::throwOnNullOrUndefined, CallResultDestination::Ignore);
-    as->checkException();
+    BASELINEJIT_GENERATE_RUNTIME_CALL(ThrowOnNullOrUndefined, CallResultDestination::Ignore);
+    LOAD_ACC();
 }
 
 void BaselineJIT::generate_GetTemplateObject(int index)
@@ -916,19 +916,17 @@ void BaselineJIT::generate_GetTemplateObject(int index)
     as->prepareCallWithArgCount(2);
     as->passInt32AsArg(index, 1);
     as->passFunctionAsArg(0);
-    BASELINEJIT_GENERATE_RUNTIME_CALL(RuntimeHelpers::getTemplateObject, CallResultDestination::InAccumulator);
-    as->checkException();
+    BASELINEJIT_GENERATE_RUNTIME_CALL(GetTemplateObject, CallResultDestination::InAccumulator);
 }
 
-void BaselineJIT::startInstruction(Instr::Type /*instr*/)
+ByteCodeHandler::Verdict BaselineJIT::startInstruction(Instr::Type /*instr*/)
 {
-    if (hasLabel())
+    if (labels.contains(currentInstructionOffset()))
         as->addLabel(currentInstructionOffset());
+    return ProcessInstruction;
 }
 
 void BaselineJIT::endInstruction(Instr::Type instr)
 {
     Q_UNUSED(instr);
 }
-
-#endif // V4_ENABLE_JIT
