@@ -3,24 +3,28 @@
 
 /*---
 esid: sec-array.prototype.indexof
-es5id: 15.4.4.14-5-26
+es5id: 15.4.4.14-5-27
 description: >
-    Array.prototype.indexOf - side effects produced by step 2 are
+    Array.prototype.indexOf - side effects produced by step 3 are
     visible when an exception occurs
 ---*/
 
-var stepTwoOccurs = false;
+var stepThreeOccurs = false;
 var stepFiveOccurs = false;
 
 var obj = {};
 
 Object.defineProperty(obj, "length", {
   get: function() {
-    stepTwoOccurs = true;
-    if (stepFiveOccurs) {
-      throw new Error("Step 5 occurred out of order");
-    }
-    return 20;
+    return {
+      valueOf: function() {
+        stepThreeOccurs = true;
+        if (stepFiveOccurs) {
+          throw new Error("Step 5 occurred out of order");
+        }
+        return 20;
+      }
+    };
   },
   configurable: true
 });
@@ -34,5 +38,5 @@ var fromIndex = {
 
 Array.prototype.indexOf.call(obj, undefined, fromIndex);
 
-assert(stepTwoOccurs, 'stepTwoOccurs !== true');
+assert(stepThreeOccurs, 'stepThreeOccurs !== true');
 assert(stepFiveOccurs, 'stepFiveOccurs !== true');
