@@ -3,10 +3,10 @@
 
 /*---
 esid: sec-array.prototype.lastindexof
-es5id: 15.4.4.15-6-4
+es5id: 15.4.4.15-7-4
 description: >
-    Array.prototype.lastIndexOf returns -1 when 'fromIndex' and
-    'length' are both 0
+    Array.prototype.lastIndexOf returns -1 when abs('fromIndex') is
+    length of array
 ---*/
 
-assert.sameValue([].lastIndexOf(1, 0), -1, '[].lastIndexOf(1, 0)');
+assert.sameValue([1, 2, 3, 4].lastIndexOf(2, -4), -1, '[1, 2, 3, 4].lastIndexOf(2, -4)');
