@@ -2,10 +2,10 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 15.2.3.11-4-6
-description: Object.isSealed returns false for all built-in objects (Array)
+es5id: 15.2.3.11-4-14
+description: Object.isSealed returns false for all built-in objects (Math)
 ---*/
 
-var b = Object.isSealed(Array);
+var b = Object.isSealed(Math);
 
 assert.sameValue(b, false, 'b');
